<?php
/**
 * @author Amasty Team
 * @copyright Copyright (c) Amasty (https://www.amasty.com)
 * @package Follow Up Email for Magento 2
 */

namespace Amasty\Followup\Block\Email\Items;

use Magento\Catalog\Api\ProductRepositoryInterface;
use Magento\Framework\View\Element\Template;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Quote\Model\Quote\Item;
use Magento\ConfigurableProduct\Model\Product\Type\Configurable;

class Upsell extends Template
{
    /**
     * @var CartRepositoryInterface
     */
    private $cartRepository;

    /**
     * @var ProductRepositoryInterface
     */
    private $productRepository;

    public function __construct(
        Template\Context $context,
        CartRepositoryInterface $cartRepository,
        ProductRepositoryInterface $productRepository,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->cartRepository = $cartRepository;
        $this->productRepository = $productRepository;
    }

    /**
     * @return array
     */
    public function getItems()
    {
        $upSellProducts = [];
        $quote = $this->cartRepository->get((int)$this->getQuoteId());

        /** @var Item $item */
        foreach ($quote->getAllVisibleItems() as $item) {
            if ($item->getProductType() === Configurable::TYPE_CODE) {
                $product = $this->productRepository->get($item->getSku(), false, $item->getQuote()->getStoreId());
            } else {
                $product = $item->getProduct();
            }

            //phpcs:ignore Magento2.Performance.ForeachArrayMerge.ForeachArrayMerge
            $upSellProducts = array_merge($upSellProducts, $product->getUpSellProducts());
        }

        return $upSellProducts;
    }
}
